#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
    /// </summary>
    [Description("T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'")]
    public class T3Average : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int    period    = 10;    // Default setting for Period
            private double overshoot = 0.700; // Default setting for Overshoot

		    private double coeff1;
		    private double coeff2;
		    private double coeff3;
		    private double coeff4;
		    		
            private DataSeries filter1;
		    private DataSeries filter2;
		    private DataSeries filter3;
		    private DataSeries filter4;
		    private DataSeries filter5;
		    private DataSeries filter6;
		    private DataSeries t3Ave;
		    
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "T3AverageUp"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "T3AverageDown"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "T3AverageNeutral"));
			
			Plots[0].Pen.Width = 3;
			Plots[1].Pen.Width = 3;
			Plots[2].Pen.Width = 3;
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= true;
			
			filter1 = new DataSeries(this);
			filter2 = new DataSeries(this);
			filter3 = new DataSeries(this);
			filter4 = new DataSeries(this);
			filter5 = new DataSeries(this);
			filter6 = new DataSeries(this);
			t3Ave   = new DataSeries(this);
			
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < Period)
				return;
			
			filter1.Set(EMA(Input, Period)[0]);
			filter2.Set(EMA(filter1, Period)[0]);
			filter3.Set(EMA(filter2, Period)[0]);
			filter4.Set(EMA(filter3, Period)[0]);
			filter5.Set(EMA(filter4, Period)[0]);
			filter6.Set(EMA(filter5, Period)[0]);
			
			coeff1 = - Overshoot * Overshoot * Overshoot;
			coeff2 = 3 * Overshoot * Overshoot + 3 * Overshoot * Overshoot * Overshoot;
			coeff3 = - 6 * Overshoot * Overshoot - 3 * Overshoot - 3 * Overshoot * Overshoot * Overshoot;
			coeff4 = 1 + 3 * Overshoot + Overshoot * Overshoot * Overshoot + 3 * Overshoot * Overshoot;
			
			t3Ave.Set(coeff1 * filter6[0] + coeff2 * filter5[0] + coeff3 * filter4[0] + coeff4 * filter3[0]); 
						
			if (Rising(t3Ave))
			{
				// Connects the rising plot segment with the other plots
				T3AverageUp.Set(1, t3Ave[1]);
				
				// Adds the new rising plot line segment to the line
				T3AverageUp.Set(t3Ave[0]);
			}
			
			else if (Falling(t3Ave))
			{
				// Connects the new falling plot segment with the rest of the line
				T3AverageDown.Set(1, t3Ave[1]);
				
				// Adds the new falling plot line segment to the line
				T3AverageDown.Set(t3Ave[0]);
			}
			
			else
			{
				// Connects the neutral plot segment with the rest of the line
				T3AverageNeutral.Set(1, t3Ave[1]);
				
				// Adds the new neutral plot line segment to the line
				T3AverageNeutral.Set(t3Ave[0]);
            }
        }  
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries T3AverageUp
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries T3AverageDown
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries T3AverageNeutral
        {
            get { return Values[2]; }
        }

        [Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(3, value); }
        }

        [Description("Controls the T3 overshoot behavior")]
        [Category("Parameters")]
        public double Overshoot
        {
            get { return overshoot; }
            set { overshoot = Math.Max(0, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private T3Average[] cacheT3Average = null;

        private static T3Average checkT3Average = new T3Average();

        /// <summary>
        /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
        /// </summary>
        /// <returns></returns>
        public T3Average T3Average(double overshoot, int period)
        {
            return T3Average(Input, overshoot, period);
        }

        /// <summary>
        /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
        /// </summary>
        /// <returns></returns>
        public T3Average T3Average(Data.IDataSeries input, double overshoot, int period)
        {
            checkT3Average.Overshoot = overshoot;
            overshoot = checkT3Average.Overshoot;
            checkT3Average.Period = period;
            period = checkT3Average.Period;

            if (cacheT3Average != null)
                for (int idx = 0; idx < cacheT3Average.Length; idx++)
                    if (Math.Abs(cacheT3Average[idx].Overshoot - overshoot) <= double.Epsilon && cacheT3Average[idx].Period == period && cacheT3Average[idx].EqualsInput(input))
                        return cacheT3Average[idx];

            T3Average indicator = new T3Average();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Overshoot = overshoot;
            indicator.Period = period;
            indicator.SetUp();

            T3Average[] tmp = new T3Average[cacheT3Average == null ? 1 : cacheT3Average.Length + 1];
            if (cacheT3Average != null)
                cacheT3Average.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheT3Average = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.T3Average T3Average(double overshoot, int period)
        {
            return _indicator.T3Average(Input, overshoot, period);
        }

        /// <summary>
        /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
        /// </summary>
        /// <returns></returns>
        public Indicator.T3Average T3Average(Data.IDataSeries input, double overshoot, int period)
        {
            return _indicator.T3Average(input, overshoot, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.T3Average T3Average(double overshoot, int period)
        {
            return _indicator.T3Average(Input, overshoot, period);
        }

        /// <summary>
        /// T3 Average by Tim Tillson, presented in TASC article 'Smoothing techniques for more accurate signals'
        /// </summary>
        /// <returns></returns>
        public Indicator.T3Average T3Average(Data.IDataSeries input, double overshoot, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.T3Average(input, overshoot, period);
        }

    }
}
#endregion
